// -*- C++ -*-
/*!
 * @file  LRFCapture_SICK.cpp
 * @brief LRF capture module for SICK
 * @date $Date$
 *
 * $Id$
 */

#include "LRFCapture_SICK.h"
#include <vector>

/// 初期化時のデバイス名
#define DEFAULT_DEV "/dev/ttyUSB0"
/// 初期化時のSICK型番 "LMS" or "PLS"
#define DEFAULT_TYPE "LMS"
/// 初期化時のRemissionの有効/無効 "direct", "normalized", "no"
#define DEFAULT_ENABLE_REM "no"
/// 初期化時の計測範囲[deg]
#define DEFAULT_FOV (180)
/// 初期化時の分解能[deg]
#define DEFAULT_RESOLUTION (0.5)
/// 初期化時のボーレート
#define DEFAULT_BAUDRATE (38400)

// Module specification
// <rtc-template block="module_spec">
static const char* lrfcapture_sick_spec[] =
  {
    "implementation_id", "LRFCapture_SICK",
    "type_name",         "LRFCapture_SICK",
    "description",       "LRF capture module for SICK",
    "version",           "1.0.0",
    "vendor",            "Tsuyoshi Suenaga, NAIST",
    "category",          "sensor",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.str_port", "/dev/ttyUSB0",
    "conf.default.distance_resolution_mode", "cm",

    ""
  };
// </rtc-template>

LRFCapture_SICK::LRFCapture_SICK(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_RangeDataOut("RangeData", m_RangeData),
    m_LRFInfoPort("LRFInfo"),
    m_LRF(&g_laser),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("RangeData", m_RangeDataOut);
  
  // Set service provider to Ports
  m_LRFInfoPort.registerProvider("LRF", "LRFInfo", m_LRF);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_LRFInfoPort);
  
  // </rtc-template>

}

LRFCapture_SICK::~LRFCapture_SICK()
{
}



RTC::ReturnCode_t LRFCapture_SICK::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("str_port", m_str_port, "/dev/ttyUSB0");
  bindParameter("distance_resolution_mode", m_distance_resolution_mode, "cm");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t LRFCapture_SICK::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LRFCapture_SICK::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LRFCapture_SICK::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t LRFCapture_SICK::onActivated(RTC::UniqueId ec_id)
{
  std::vector<char> dev_port(m_str_port.begin(), m_str_port.end());
  dev_port.push_back('\0');

  // SICKの初期化
  sick_set_default_parameters(&g_laser,0);

  // 距離分解能のモードを変更する
  if(m_distance_resolution_mode == "cm")
    g_laser.settings.range_res = CM;
  else if(m_distance_resolution_mode == "mm")
    g_laser.settings.range_res = MM;
  else if(m_distance_resolution_mode == "dm")
    g_laser.settings.range_res = DM;
  else{
    std::cerr << "'distance_resolution_mode' should be 'cm', 'mm' or 'dm'." << std::endl;
    std::exit(1);
  }

  // SICKのモード設定
  sick_parse_params(&g_laser, &dev_port[0], DEFAULT_TYPE,
		    DEFAULT_RESOLUTION, DEFAULT_ENABLE_REM,
		    DEFAULT_FOV, DEFAULT_BAUDRATE);
  // SICKのモード設定が異常に遅いため，SICK起動直後の最初の1回はここで必ず失敗する．
  // Ctrl-c で強制終了してから10秒程度待ってから起動させると問題なく動作する．

  sick_start_laser(&g_laser);
  
  return RTC::RTC_OK;
}



RTC::ReturnCode_t LRFCapture_SICK::onDeactivated(RTC::UniqueId ec_id)
{
  sick_stop_laser(&g_laser);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t LRFCapture_SICK::onExecute(RTC::UniqueId ec_id)
{
  // SICKから距離データを取得
  sick_handle_laser(&g_laser);
  if( g_laser.new_reading){
    m_RangeData.data.length(g_laser.numvalues);
    for(int i=0; i<g_laser.numvalues; i++)
      m_RangeData.data[i] = g_laser.range[i]*10;
    std::cout << m_RangeData.data[0] << std::endl;
    // データ送信
    m_RangeDataOut.write();
  }
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t LRFCapture_SICK::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LRFCapture_SICK::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LRFCapture_SICK::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LRFCapture_SICK::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LRFCapture_SICK::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void LRFCapture_SICKInit(RTC::Manager* manager)
  {
    RTC::Properties profile(lrfcapture_sick_spec);
    manager->registerFactory(profile,
                             RTC::Create<LRFCapture_SICK>,
                             RTC::Delete<LRFCapture_SICK>);
  }
  
};


